from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from activity.models import Process
from activity.models import Activity


@admin.register(Process)
class ProcessAdmin(admin.ModelAdmin):
    class Meta:
        verbose_name_plural = _('Processes')


@admin.register(Activity)
class ActivityAdmin(admin.ModelAdmin):
    class Meta:
        verbose_name_plural = _('Activities')



from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from activity.constants import K_TRIGGER_PROCESS_ON_CREATE
from activity.constants import K_TRIGGER_PROCESS_ON_UPDATE
from activity.constants import K_TRIGGER_ACTIVITY_ON_COMPLETE
from activity.constants import K_TRIGGER_ACTIVITY_ON_CREATE
from activity.constants import K_TRIGGER_ACTIVITY_ON_UPDATE
from activity.constants import K_TRIGGER_ACTIVITY_ON_DELETE
from activity.constants import K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT
from activity.constants import K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT
from activity.constants import K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND
from activity.services import BaseWebhookService


class BaseWebhookView(APIView):
    webhook_service = None

    def get_webhook_service(self):
        if self.webhook_service is None:
            raise NotImplementedError('No webhook service')
        if not issubclass(type(self.webhook_service), BaseWebhookService):
            raise ValueError(
                'Webhook service not inherit from BaseWebhookService')
        return self.webhook_service

    def perform_on_process_created(self, request):
        return self.get_webhook_service() \
            .on_process_created(request.data['process'])

    def perform_on_process_updated(self, request):
        return self.get_webhook_service() \
            .on_process_updated(request.data['process'])

    def perform_on_activity_completed(self, request):
        return self.get_webhook_service() \
            .on_activity_completed(request.data['activity'])

    def perform_on_activity_created(self, request):
        return self.get_webhook_service() \
            .on_activity_created(request.data['activity'])

    def perform_on_activity_updated(self, request):
        return self.get_webhook_service() \
            .on_activity_updated(request.data['activity'])

    def perform_on_deleted(self, request):
        return self.get_webhook_service() \
            .on_activity_deleted(request.data['activity'])

    def perform_on_submission_submitted(self, request):
        return self.get_webhook_service() \
            .on_activity_submission_submitted(
            request.data['activity_submission'], request.data['activity'])

    def perform_on_submission_unsubmitted(self, request):
        return self.get_webhook_service() \
            .on_activity_submission_unsubmitted(
            request.data['activity_submission'], request.data['activity'])

    def perform_on_activities_reminder_sent(self, request):
        return self.get_webhook_service() \
            .on_activities_reminder_sent(
            request.data['activities_by_remaining_days'])

    def perform_on_value_error(self, error):
        return self.get_webhook_service() \
            .on_value_error(error)

    def perform_on_internal_server_error(self, error, request_data):
        return self.get_webhook_service() \
            .on_internal_server_error(error, request_data)

    def validate_trigger_type(self, request):
        if 'trigger_type' not in request.data:
            raise ValueError({'trigger_type': 'missing trigger_type'})
        return request.data['trigger_type']

    def post(self, request, *args, **kwargs):
        try:
            trigger_type = self.validate_trigger_type(request)

            if trigger_type == K_TRIGGER_PROCESS_ON_CREATE:
                res = self.perform_on_process_created(request)
            elif trigger_type == K_TRIGGER_PROCESS_ON_UPDATE:
                res = self.perform_on_process_updated(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_COMPLETE:
                res = self.perform_on_activity_completed(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_CREATE:
                res = self.perform_on_activity_created(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_UPDATE:
                res = self.perform_on_activity_updated(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_ON_DELETE:
                res = self.perform_on_deleted(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT:
                res = self.perform_on_submission_submitted(request)
            elif trigger_type == K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT:
                res = self.perform_on_submission_unsubmitted(request)
            elif trigger_type == K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND:
                res = self.perform_on_activities_reminder_sent(request)
            else:
                raise ValueError({'trigger_type': 'invalid trigger_type'})
            return Response(res, status.HTTP_200_OK)
        except ValueError as e:
            self.perform_on_value_error(e)
            return Response(e, status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            self.perform_on_internal_server_error(e, request.data)
            return Response({'msg': str(e)}, status.HTTP_500_INTERNAL_SERVER_ERROR)



K_TRIGGER_PROCESS_ON_CREATE = 'PROCESS_ON_CREATE'
K_TRIGGER_PROCESS_ON_UPDATE = 'PROCESS_ON_UPDATE'
K_TRIGGER_ACTIVITY_ON_CREATE = 'ACTIVITY_ON_CREATE'
K_TRIGGER_ACTIVITY_ON_UPDATE = 'ACTIVITY_ON_UPDATE'
K_TRIGGER_ACTIVITY_ON_COMPLETE = 'ACTIVITY_ON_COMPLETE'
K_TRIGGER_ACTIVITY_ON_DELETE = 'ACTIVITY_ON_DELETE'
K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT = 'ACTIVITY_SUBMISSION_ON_SUBMIT'
K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT = 'ACTIVITY_SUBMISSION_ON_UNSUBMIT'
K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND = 'ACTIVITIES_REMINDER_ON_SEND'


K_SUBMISSION_TYPE_NOTE = 'Note'
K_SUBMISSION_TYPE_NOTE_AND_DATE = 'NoteAndDate'
K_SUBMISSION_TYPE_NOTE_AND_MULTIPLE_FILE = 'NoteAndMultipleFile'
K_SUBMISSION_TYPE_NOTE_AND_YES_NO = 'NoteAndYesNo'

K_PROCESS_STATUS_COMPLETED = 'Completed'
K_PROCESS_STATUS_CANCELLED = 'Cancelled'
K_PROCESS_STATUS_ON_HOLD = 'On Hold'
K_PROCESS_STATUS_ON_PROGRESS = 'On Progress'
K_PROCESS_STATUS_IGNORED = 'Ignored'



import logging

from activity.api import ActivityAPI

logger = logging.getLogger('Activity API Client')


# Activity API Client Python Interface #
class ActivityAPIClient(object):
    """
    Interface for interact with Ronan activity service
    """
    ALL = 'All'
    RESTRICTED = 'Restricted'

    def __init__(self):
        self.api = ActivityAPI()

    def create_process(self, process_type, external_key, status, additional_data={}):
        """
        Create process to Ronan activity service and
        return response's content. Raise ClientError when request is failed.
        :param process_type: string process_type that defined in Ronan
        :param external_key: string to identify process from consumer service (example: Project ID)
        :param status: enum(['On Hold', 'On Progress', 'Completed', 'Cancelled'])
        :param additional_data: dict()
        :return: dict
        {
            "processType": "...",
            "externalKey": "...",
            "status": "...",
            "additionalData": "..."
        }
        """
        uri = '/processes'
        data = {
            'processType': process_type,
            'externalKey': external_key,
            'status': status,
            'additionalData': additional_data
        }
        return self.api.post(uri, data=data)

    def update_process(self, process_key, **kwargs):
        """
        Update process to Ronan activity service and
        return response's content. Raise ClientError when request is failed.
        :param process_type: string process_type that defined in Ronan
        :param external_key: string to identify process from consumer service (example: Project ID)
        :param status: enum(['On Hold', 'On Progress', 'Completed', 'Cancelled'])
        :return: dict
        {
            "processType": "...",
            "externalKey": "...",
            "status": "..."
        }
        """
        process_type = kwargs.get('process_type', None)
        external_key = kwargs.get('external_key', None)
        status = kwargs.get('status', None)
        additional_data = kwargs.get('additional_data', None)

        data = {}
        if process_type:
            data['processType'] = process_type
        if external_key:
            data['externalKey'] = external_key
        if status:
            data['status'] = status
        if additional_data:
            data['additionalData'] = additional_data

        uri = '/processes/%s' % str(process_key)
        return self.api.patch(uri, data=data)

    def create_activity(self, process_key, template_slug, additional_data=None,
                        **activity_data):
        """
        Create activity based on template activity slug given.
        Return response's content. Raise ClientError when request is failed.
        :param process_key: string to identify process that running in Ronan. You have to create Process first
                            before start an activity.
        :param template_slug: string slug based on template name to identify the template (example: TEMPLATE_NAME_SLUG).
        :param additional_data: dict contains optional data that you want to carry.
        :param activity_data: dict contains optional activity data.
                              You may want to override the default value of the template.
        :return: dict
        """
        uri = '/processes/%s/activities' % process_key
        data = activity_data.copy()
        data.update({
            'fromTemplateActivity': template_slug,
        })

        if additional_data is not None:
            data['additionalData'] = additional_data

        return self.api.post(uri, data=data)

    def update_activity(self, process_key, activity_id, **activity_data):
        """
        Update activity. Return response's content. Raise ClientError when request is failed.
        :param process_key: string to identify process that running in Ronan. You have to create Process first
                            before start an activity.
        :param activity_id: int to identify the activity
        :param activity_data: dict
        :return: dict
        """
        uri = '/processes/%s/activities/%s' % (process_key, activity_id)
        return self.api.patch(uri, data=activity_data)

    def delete_activity(self, process_key, activity_id):
        """
        Delete activity. Return response's content. Raise ClientError when request is failed.
        :param process_key: string to identify process that running in Ronan. You have to create Process first
                            before start an activity.
        :param activity_id: int to identify the activity
        :return: dict
        """
        uri = '/processes/%s/activities/%s' % (process_key, activity_id)
        return self.api.delete(uri)

    def submit_activity_submission(self, process_key, activity_id, activity_submission_id, **activity_submission_data):
        """
        Submit activity submission. Return response's content. Raise ClientError when request is failed.
        :param process_key: string to identify process that running in Ronan. You have to create Process first
                            before start an activity and an activity submission.
        :param activity_id: int to identify the activity
        :param activity_submission_id: int to identify the activity submission
        :param activity_submission_data: dict
        :return: dict
        """
        uri = '/processes/%s/activities/%s/submissions/%s/submit' % (process_key, activity_id, activity_submission_id)
        return self.api.patch(uri, data=activity_submission_data)

    def unsubmit_activity_submission(self, process_key, activity_id, activity_submission_id, **activity_submission_data):
        """
        Un-submit activity submission. Return response's content. Raise ClientError when request is failed.
        :param process_key: string to identify process that running in Ronan. You have to create Process first
                            before start an activity and an activity submission.
        :param activity_id: int to identify the activity
        :param activity_submission_id: int to identify the activity submission
        :param activity_submission_data: dict
        :return: dict
        """
        uri = '/processes/%s/activities/%s/submissions/%s/unsubmit' % (process_key, activity_id, activity_submission_id)
        return self.api.patch(uri, data=activity_submission_data)

    def retrieve_template_activity(self, template_slug):
        """
        Retreive template activity and return response's content.
        Raise ClientError when request is failed.
        :param template_slug: string slug based on template name to identify the template (example: TEMPLATE_NAME_SLUG)
        :return: dict
        """
        uri = '/template-activities/%s' % template_slug
        return self.api.get(uri)

    def retrieve_process(self, process_key):
        """
        Retrieve details of a single process and return response's content.
        Raise ClientError when request is failed
        :param process_key: string to identify process that running in Ronan
        :return: dict
        """
        uri = '/processes/%s' & process_key
        return self.api.get(uri)

    def retrieve_process_list(self):
        """
        Retrieve list of process and return response's content.
        Raise ClientError when request is failed
        :return: dict
        """
        uri = '/processes'
        return self.api.get(uri)

    def retrieve_process_activity_list(self, process_key, access_level):
        """
        Retrieve list of process and return response's content.
        Raise ClientError when request is failed
        :return: dict
        """
        uri = '/processes/%s/activities?access_level=%s' % (
            process_key, access_level)
        return self.api.get(uri)

    def retrieve_activity_list(self, activity_ids):
        """
        Retrieve list of activity with id included in activity_ids and return response's content.
        Raise ClientError when request is failed
        :return: dict
        """
        activity_ids_params = '&'.join(
            ['activityIds=%d' % id for id in activity_ids]
        )
        uri = '/activities?%s' % activity_ids_params
        return self.api.get(uri)

    def search(self, index, query="", facets=[], disjunctive_facets_refinements={}, ordering=[], hits_per_page=20, page=1):
        """
        Retrieve list of object with filter specified.
        :return: dict
        """
        uri = '/search/'
        data = {
            'index': index,
            'query': query,
            'facets': facets,
            'disjunctive_facets_refinements': disjunctive_facets_refinements,
            'ordering': ordering,
            'hits_per_page': hits_per_page,
            'page': page,
        }
        return self.api.post(uri, data=data)



import os
import json

import requests
from rest_framework import status


K_API_BASE_URL = os.environ['ACTIVITY_BASE_API_URL']
K_API_KEY = os.environ['ACTIVITY_API_KEY']


class ActivityAPI(object):
    BASE_URL = K_API_BASE_URL

    def get_headers(self, original_headers={}):
        headers = original_headers.copy()
        headers['content-type'] = 'application/json'
        headers['api-key'] = K_API_KEY
        return headers

    def get(self, endpoint, params=None, headers={}):
        try:
            req = requests.get(self.BASE_URL + endpoint, params=params,
                               headers=self.get_headers(headers))
            req.raise_for_status()
            return json.loads(req.content)
        except requests.exceptions.HTTPError as e:
            raise ClientError(req.status_code, req.content, str(e))
        except Exception as e:
            raise ClientError(status.HTTP_408_REQUEST_TIMEOUT,
                              'Terjadi Kesalahan Pada Sistem, Silakan Mencoba Kembali', str(e))

    def post(self, endpoint, data={}, headers={}):
        try:
            req = requests.post(self.BASE_URL + endpoint, json=data,
                                headers=self.get_headers(headers))
            req.raise_for_status()
            return json.loads(req.content)
        except requests.exceptions.HTTPError as e:
            raise ClientError(req.status_code, req.content, str(e))
        except Exception as e:
            raise ClientError(status.HTTP_408_REQUEST_TIMEOUT,
                              'Terjadi Kesalahan Pada Sistem, Silakan Mencoba Kembali', str(e))

    def patch(self, endpoint, data={}, headers={}):
        try:
            req = requests.patch(self.BASE_URL + endpoint, json=data,
                                 headers=self.get_headers(headers))
            return json.loads(req.content)
        except requests.exceptions.HTTPError as e:
            raise ClientError(req.status_code, req.content, str(e))
        except Exception as e:
            raise ClientError(status.HTTP_408_REQUEST_TIMEOUT,
                              'Terjadi Kesalahan Pada Sistem, Silakan Mencoba Kembali', str(e))

    def delete(self, endpoint, headers={}):
        try:
            req = requests.delete(self.BASE_URL + endpoint, headers=self.get_headers(headers))
            req.raise_for_status()
            return json.loads(req.content)
        except requests.exceptions.HTTPError as e:
            raise ClientError(req.status_code, req.content, str(e))
        except Exception as e:
            raise ClientError(status.HTTP_408_REQUEST_TIMEOUT,
                              'Terjadi Kesalahan Pada Sistem, Silakan Mencoba Kembali', str(e))

    def check_base_url(self):
        if self.BASE_URL is None:
            raise Exception('Base URL is not specified')


class ClientError(Exception):

    def __init__(self, code, message, error):
        Exception.__init__(self, "%d: %s - %s" % (code, message, error))
        self.code = code
        self.message = message
        self.error = error
